package com.upyuns.platform.rs.website.biz;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.bean.copier.CopyOptions;
import com.alibaba.fastjson.JSONObject;
import com.github.wxiaoqi.security.admin.feign.UserFeign;
import com.github.wxiaoqi.security.admin.feign.dto.AppUserDTO;
import com.github.wxiaoqi.security.common.msg.ObjectRestResponse;
import com.github.wxiaoqi.security.common.util.OrderUtil;
import com.github.wxiaoqi.security.common.util.Query;
import com.github.wxiaoqi.security.common.util.process.ResultCode;
import com.github.wxiaoqi.security.common.vo.PageDataVO;
import com.upyuns.platform.rs.datacenter.fegin.DatacenterFeign;
import com.upyuns.platform.rs.datacenter.pojo.ImageDataVO;
import com.upyuns.platform.rs.universal.constant.DictionaryKey;
import com.upyuns.platform.rs.universal.entity.Dictionary;
import com.upyuns.platform.rs.universal.feign.ThirdFeign;
import com.upyuns.platform.rs.website.dto.*;
import com.upyuns.platform.rs.website.entity.*;
import com.upyuns.platform.rs.website.mapper.OrderItemMapper;
import com.upyuns.platform.rs.website.type.OrderInvoiceStatusEnum;
import com.upyuns.platform.rs.website.type.OrderShippingStatusEnum;
import com.upyuns.platform.rs.website.type.OrderStatusEnum;
import com.upyuns.platform.rs.website.type.OrderTypeEnum;
import com.upyuns.platform.rs.website.vo.ItemInfoVo;
import org.apache.commons.lang3.StringUtils;
import org.apache.regexp.RE;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.upyuns.platform.rs.website.mapper.OrderInfoMapper;
import com.github.wxiaoqi.security.common.biz.BaseBiz;
import tk.mybatis.mapper.entity.Example;

import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicReference;
import java.util.stream.Collectors;

/**
 * 订单信息
 *
 * @author jiaoruizhen
 * @email jiaoruizhen@126.com
 * @date 2020-12-03 13:29:23
 */
@Service
public class OrderInfoBiz extends BaseBiz<OrderInfoMapper, OrderInfo> {

	@Autowired
	OrderItemBiz orderItemBiz;
	@Autowired
	IndustryApplicationInfoBiz industryApplicationInfoBiz;

	@Autowired
	ImageImgStorageBiz imageImgStorageBiz;

	@Autowired
	ShoppingCartInfoBiz shoppingCartInfoBiz;

	@Autowired
	ImageInfoRelationBiz imageInfoRelationBiz;

	@Autowired
	ThirdFeign thirdFeign;

	@Autowired
	UserFeign userFeign;

	@Autowired
	HttpServletRequest request;

	@Autowired
	UserAddressBiz userAddressBiz;

	@Autowired
	OrderEInvoiceBiz orderEInvoiceBiz;

	@Autowired
	UserInvoiceBiz userInvoiceBiz;

	@Autowired
	DatacenterFeign datacenterFeign;

	@Autowired
	CustomFormBiz customFormBiz;

	/**
	 * 添加订单
	 *
	 * @param orderInfo
	 * @return
	 */
	public ObjectRestResponse addUpdate(OrderInfo orderInfo) {
		if (orderInfo == null) {
			return ObjectRestResponse.paramIsEmpty();
		}
		if (orderInfo.getOrderId() == null) {
			insertSelectiveRe(orderInfo);
		} else {
			OrderInfo old = selectById(orderInfo.getOrderId());
			BeanUtil.copyProperties(orderInfo, old, CopyOptions.create().setIgnoreNullValue(true).setIgnoreError(true));
			updateSelectiveByIdRe(old);
		}
		return ObjectRestResponse.succ();
	}

	/**
	 * 下单
	 *
	 * @param orderInfo
	 * @return
	 */
	public ObjectRestResponse generateOrder(OrderInfo orderInfo) {
		if (orderInfo == null) {
			return ObjectRestResponse.paramIsEmpty();
		}
		if (StringUtils.isBlank(orderInfo.getOrderNo())) {
			orderInfo.setOrderNo(OrderUtil.GetOrderNumber("", OrderUtil.APP_MID));
		}
		OrderInfo old = getOneByOrderNo(orderInfo.getOrderNo());
		while (old != null) {
			orderInfo.setOrderNo(OrderUtil.GetOrderNumber("", OrderUtil.APP_MID));
			old = getOneByOrderNo(orderInfo.getOrderNo());
		}
		return addUpdate(orderInfo);
	}

	/**
	 * 根据订单号查询订单信息
	 *
	 * @param orderNo
	 * @return
	 */
	public OrderInfo getOneByOrderNo(String orderNo) {
		Example example = new Example(OrderInfo.class);
		example.createCriteria().andEqualTo("orderNo", orderNo).andNotEqualTo("status", -1).andNotEqualTo("status", -2);
		return mapper.selectOneByExample(example);
	}

	/**
	 * 查询用户所有订单
	 *
	 * @param userOrderDto
	 * @return
	 */
	public ObjectRestResponse getUserOrder(UserOrderDto userOrderDto) {
		AppUserDTO appUserDTO = userFeign.userDetailByToken(request.getHeader("Authorization")).getData();
		if (appUserDTO == null) {
			return ObjectRestResponse.createFailedResult(ResultCode.RSTOKEN_EXPIRED_CODE, ResultCode.getMsg(ResultCode.RSTOKEN_EXPIRED_CODE));
		}
		appUserDTO.setUserid(appUserDTO.getUserid());
		Query query = new Query(userOrderDto);
		Example example = new Example(OrderInfo.class);
		Example orderItemExample = new Example(OrderItem.class);
		Example.Criteria orderInfoCriteria = example.createCriteria();
		Example.Criteria orderItemCriteria = orderItemExample.createCriteria();
		if (userOrderDto.getStatus() != null) {
			orderInfoCriteria.andEqualTo("status", userOrderDto.getStatus());
		}
		if (userOrderDto.getPayStatus() != null) {
			orderInfoCriteria.andEqualTo("payStatus", userOrderDto.getPayStatus());
		}
		if (userOrderDto.getUserId() != null) {
			orderInfoCriteria.andEqualTo("userId", userOrderDto.getUserId());
		}
		if (StringUtils.isNotBlank(userOrderDto.getName())) {
			orderItemCriteria.andLike("name", "%" + userOrderDto.getName() + "%");
		}
		if (StringUtils.isNotBlank(userOrderDto.getStartTime()) && StringUtils.isNotBlank(userOrderDto.getEndTime())) {
			orderItemCriteria.andBetween("crtTime", userOrderDto.getStartTime(), userOrderDto.getEndTime());
		}
		orderInfoCriteria.andNotEqualTo("status", -1);
		example.orderBy("updTime").desc();
		PageDataVO<OrderInfo> pageDataVO = PageDataVO.pageInfo(query, () -> mapper.selectByExample(example));

		if (pageDataVO != null && pageDataVO.getData() != null) {
			fillOrderItem(pageDataVO.getData());
			fillOrderEInvoice(pageDataVO.getData());
			return ObjectRestResponse.succ(pageDataVO);
		}
		return ObjectRestResponse.succ(new PageDataVO<>());
	}

	/**
	 * 填充订单发票信息
	 *
	 * @param orderInfoList
	 */
	private void fillOrderEInvoice(List<OrderInfo> orderInfoList) {
		List<Integer> orderInvoiceIdList = orderInfoList.stream().map(OrderInfo::getInvoiceId).collect(Collectors.toList());
		if (orderInvoiceIdList != null && orderInvoiceIdList.size() > 0) {
			List<OrderEInvoice> orderEInvoiceList = orderEInvoiceBiz.getOrderEInvoice(orderInvoiceIdList);
			if (orderEInvoiceList != null && orderEInvoiceList.size() > 0) {
				Map<Integer, List<OrderEInvoice>> map = orderEInvoiceList.stream().collect(Collectors.groupingBy(OrderEInvoice::getId));
				if (map != null && map.size() > 0) {
					orderInfoList.parallelStream().forEach(orderInfo -> {
						orderInfo.setOrderEInvoice(map.get(orderInfo.getInvoiceId()).get(0));
					});
				}
			}
		}
	}

	/**
	 * 查询订单详情
	 *
	 * @param orderId
	 * @return
	 */
	public OrderInfo getOrderDetail(Long orderId) {
		Example example = new Example(OrderInfo.class);
		example.createCriteria().andEqualTo("orderId", orderId).andNotEqualTo("status", -1).andNotEqualTo("status", -2);
		OrderInfo orderInfo = mapper.selectOneByExample(example);
		if (orderInfo == null) {
			return null;
		}
		if (orderInfo.getInvoiceId() != null) {
			OrderEInvoice orderEInvoice = orderEInvoiceBiz.selectById(orderInfo.getInvoiceId());
			if (orderEInvoice != null) {
				orderInfo.setOrderEInvoice(orderEInvoice);
			}
		}
		List<OrderItem> orderItemList = orderItemBiz.getByOrderId(orderInfo.getOrderId());
		if (orderItemList != null && orderItemList.size() > 0) {
			orderInfo.setOrderItemList(orderItemList);
		}
		return orderInfo;
	}

	/**
	 * 填充订单商品
	 *
	 * @param orderInfoList
	 */
	private void fillOrderItem(List<OrderInfo> orderInfoList) {
		List<Long> orderIdList = orderInfoList.stream().map(OrderInfo::getOrderId).collect(Collectors.toList());
		List<OrderItem> orderItemList = orderItemBiz.getAllByOrderId(orderIdList);
		if (orderItemList != null) {
			Map<Long, List<OrderItem>> map = orderItemList.stream().collect(Collectors.groupingBy(OrderItem::getOrderId));
			if (map != null && map.size() > 0) {
				orderInfoList.parallelStream().forEach(orderInfo -> {
					orderInfo.setOrderItemList(map.get(orderInfo.getOrderId()));
				});
			}
		}
	}

	/**
	 * 提交订单
	 *
	 * @param orderInfoDto
	 * @return
	 */
	public ObjectRestResponse submitOrder(OrderInfoDto orderInfoDto) {
		AppUserDTO appUserDTO = userFeign.userDetailByToken(request.getHeader("Authorization")).getData();
		if (appUserDTO == null) {
			return ObjectRestResponse.createFailedResult(ResultCode.RSTOKEN_EXPIRED_CODE, ResultCode.getMsg(ResultCode.RSTOKEN_EXPIRED_CODE));
		}

		if (orderInfoDto == null || orderInfoDto.getItemInfoDtoList() == null) {
			return ObjectRestResponse.paramIsEmpty();
		}
		List<ItemInfoDto> itemInfoDtoList = orderInfoDto.getItemInfoDtoList();
		List<OrderItem> orderItemList = new ArrayList<>();
		AtomicReference<Integer> number = new AtomicReference<>(0);
		AtomicReference<Double> amount = new AtomicReference<>((double) 0);
		itemInfoDtoList.parallelStream().forEach(itemInfoDto -> {
			switch (OrderTypeEnum.getByCode(itemInfoDto.getType())) {
				//标准数据
				case STANDARD_DATA:
					List<ImageDataVO> imageDataVOList = datacenterFeign.queryByIds(orderInfoDto.getDetailId() + "").getData();
					if (imageDataVOList != null && imageDataVOList.size() > 0) {
						ImageDataVO imageDataVO = imageDataVOList.get(0);
						OrderItem orderItem = new OrderItem();
						orderItem.setType(itemInfoDto.getType());
						orderItem.setTotalAmount(imageDataVO.getPrice().multiply(new BigDecimal(itemInfoDto.getNumber())));
						orderItem.setPrice(imageDataVO.getPrice());
						orderItem.setNumber(itemInfoDto.getNumber());
						orderItem.setDetailJson(JSONObject.toJSONString(imageDataVO));
						orderItem.setDetailId(Long.valueOf(imageDataVO.getId()));
						orderItemList.add(orderItem);
						//总数量
						number.updateAndGet(v -> v + itemInfoDto.getNumber());
						BigDecimal totalAmount = imageDataVO.getPrice().multiply(new BigDecimal(itemInfoDto.getNumber()));
						amount.updateAndGet(v -> new Double((double) (v + totalAmount.doubleValue())));
					}
					break;
				//行业应用
				case INDUSTRY_INFO:
					IndustryApplicationInfo industryApplicationInfo = industryApplicationInfoBiz.getOneById(itemInfoDto.getId().intValue()).getData();
					if (industryApplicationInfo != null) {
						OrderItem orderItem = new OrderItem();
						orderItem.setType(itemInfoDto.getType());
						orderItem.setTotalAmount(industryApplicationInfo.getPrice().multiply(new BigDecimal(itemInfoDto.getNumber())));
						orderItem.setPrice(industryApplicationInfo.getPrice());
						orderItem.setNumber(itemInfoDto.getNumber());
						orderItem.setName(industryApplicationInfo.getTitle());
						orderItem.setItemPic(industryApplicationInfo.getCoverImg());
						orderItem.setDetailJson(JSONObject.toJSONString(industryApplicationInfo));
						orderItem.setDetailId(Long.valueOf(industryApplicationInfo.getId()));
						orderItemList.add(orderItem);
						//总数量
						number.updateAndGet(v -> v + itemInfoDto.getNumber());
						BigDecimal totalAmount = industryApplicationInfo.getPrice().multiply(new BigDecimal(itemInfoDto.getNumber()));
						amount.updateAndGet(v -> new Double((double) (v + totalAmount.doubleValue())));
					}
					break;
				//影像图库
				case IMAGE_STORAGE:
					ImageInfoRelation imageInfoRelation = imageInfoRelationBiz.selectById(itemInfoDto.getId());
					if (imageInfoRelation != null) {
						ImageImgStorage imageImgStorage = imageImgStorageBiz.getDetail(itemInfoDto.getId().intValue()).getData();
						if (imageImgStorage != null) {
							List<ImageInfoRelation> imageInfoRelationList = new ArrayList<>();
							imageInfoRelationList.add(imageInfoRelation);
							imageImgStorage.setImageInfoRelationList(imageInfoRelationList);
							OrderItem orderItem = new OrderItem();
							orderItem.setType(itemInfoDto.getType());
							orderItem.setTotalAmount(imageInfoRelation.getPrice().multiply(new BigDecimal(itemInfoDto.getNumber())));
							orderItem.setPrice(imageInfoRelation.getPrice());
							orderItem.setNumber(itemInfoDto.getNumber());
							orderItem.setName(imageImgStorage.getName());
							orderItem.setItemPic(imageImgStorage.getCoverImg());
							orderItem.setDetailJson(JSONObject.toJSONString(imageImgStorage));
							orderItem.setDetailId(Long.valueOf(imageImgStorage.getId()));
							orderItemList.add(orderItem);
							//总数量
							number.updateAndGet(v -> v + itemInfoDto.getNumber());
							BigDecimal totalAmount = imageInfoRelation.getPrice().multiply(new BigDecimal(itemInfoDto.getNumber()));
							amount.updateAndGet(v -> new Double((double) (v + totalAmount.doubleValue())));
						}
					}
					break;
				//购物车
				case SHOPPING_CART_INFO:
					ShoppingCartInfo shoppingCartInfo = shoppingCartInfoBiz.selectById(itemInfoDto.getId());
					if (shoppingCartInfo != null) {
						OrderItem orderItem = new OrderItem();
						orderItem.setType(itemInfoDto.getType());
						orderItem.setTotalAmount(shoppingCartInfo.getPrice().multiply(new BigDecimal(itemInfoDto.getNumber())));
						orderItem.setPrice(shoppingCartInfo.getPrice());
						orderItem.setNumber(itemInfoDto.getNumber());
						orderItem.setName(shoppingCartInfo.getName());
						orderItem.setItemPic(shoppingCartInfo.getItemPic());
						orderItem.setDetailJson(shoppingCartInfo.getDetailJson());
						orderItem.setDetailId(shoppingCartInfo.getDetailId());
						orderItemList.add(orderItem);
						number.updateAndGet(v -> v + itemInfoDto.getNumber());
						BigDecimal totalAmount = shoppingCartInfo.getTotalAmount();
						amount.updateAndGet(v -> new Double((double) (v + totalAmount.doubleValue())));
						//下单后删除购物车
						shoppingCartInfo.setIsDel(1);
						shoppingCartInfoBiz.updateSelectiveByIdRe(shoppingCartInfo);
					}
					break;
				default:
					break;
			}
		});

		OrderInfo orderInfo = new OrderInfo();
		orderInfo.setRealName(appUserDTO.getRealname());
		orderInfo.setUserId(appUserDTO.getUserid());
		orderInfo.setPhone(appUserDTO.getUsername());
		orderInfo.setNickname(appUserDTO.getNickname());
		orderInfo.setHeadImg(appUserDTO.getHeadimgurl());
		if (orderInfoDto.getUserAddressId() != null) {
			UserAddress userAddress = userAddressBiz.selectById(orderInfoDto.getUserAddressId());
			if (userAddress != null) {
				orderInfo.setReceiveName(userAddress.getConsigneeName());
				orderInfo.setReceivePhone(userAddress.getConsigneePhone());
				orderInfo.setReceiveAddress(userAddress.getProvince() + userAddress.getCity() + userAddress.getTown() + userAddress.getConsigneeAddress());
			}
		}
		orderInfo.setRemark(orderInfoDto.getRemark());
		orderInfo.setAmount(new BigDecimal(amount.get()));
		orderInfo.setNumber(number.get());
		Dictionary dictionary = thirdFeign.findDictionaryByTypeAndCode(DictionaryKey.APP_ORDER, DictionaryKey.SEND_FEE);
		if (dictionary != null && StringUtils.isNotBlank(dictionary.getDetail())) {
			orderInfo.setFee(new BigDecimal(dictionary.getDetail()));
		}
		if (orderInfo.getFee() != null) {
			orderInfo.setTotalAmount(orderInfo.getAmount().add(orderInfo.getFee()));
		}

		//订单发票信息
		if (orderInfoDto.getUserInvoiceId() != null) {
			UserInvoice userInvoice = userInvoiceBiz.selectById(orderInfoDto.getUserInvoiceId());
			if (userInvoice != null) {
				OrderEInvoice orderEInvoice = new OrderEInvoice();
				orderEInvoice.setAmount(orderInfo.getAmount());
				orderEInvoice.setType(1);
				orderEInvoice.setOrderIds(orderInfo.getOrderId() + "");
				orderEInvoice.setTitleType(userInvoice.getTitleType());
				orderEInvoice.setTitleName(userInvoice.getTitleName());
				orderEInvoice.setTaxCode(userInvoice.getTaxCode());
				orderEInvoice.setContentRemark(orderInfoDto.getRemark());
				orderEInvoice.setContentAccount(userInvoice.getOpenBank() + "  " + userInvoice.getAccount());
				orderEInvoice.setContentContact(orderInfo.getReceiveAddress() + "  " + orderInfo.getReceiveName() + "  " + orderInfo.getReceivePhone());
				//发票内容
				if (StringUtils.isNotBlank(orderInfoDto.getInvoiceContent())) {
					orderEInvoice.setContentTitle(orderInfoDto.getInvoiceContent());
				}
				orderEInvoiceBiz.addUpdate(orderEInvoice);
				orderInfo.setInvoiceStatus(2);
				orderInfo.setInvoiceId(orderEInvoice.getId());
			}
		}

		orderInfo.setSendType(orderInfoDto.getSendType());
		orderInfo.setType(1);
		/**
		 * 新增订单
		 */
		orderInfo.setOrderNo(OrderUtil.GetOrderNumber("", OrderUtil.APP_MID));
		generateOrder(orderInfo);
		orderItemList.parallelStream().forEach(orderItem -> {
			if (orderItem != null) {
				orderItem.setOrderId(orderInfo.getOrderId());
				orderItemBiz.insertSelective(orderItem);
			}
		});
		return ObjectRestResponse.succ();
	}

	/**
	 * 确认收货
	 *
	 * @return
	 */
	public ObjectRestResponse confirmReceived(Long orderId) {
		if (orderId == null) {
			return ObjectRestResponse.paramIsEmpty();
		}
		OrderInfo orderInfo = selectById(orderId);
		if (orderInfo == null || orderInfo.getStatus().equals(OrderStatusEnum.ORDER_DELETE.getType())) {
			return ObjectRestResponse.createFailedResult(ResultCode.ORDER_NOT_EXIST, ResultCode.getMsg(ResultCode.ORDER_NOT_EXIST));
		}
		if (!orderInfo.getStatus().equals(OrderStatusEnum.ORDER_SEND.getType())) {
			return ObjectRestResponse.createFailedResult(ResultCode.FAILED_CODE, "订单状态错误");
		}
		orderInfo.setShippingStatus(OrderShippingStatusEnum.ORDER_RECEIVED.getType());
		orderInfo.setStatus(OrderStatusEnum.ORDER_RECEIVED.getType());
		updateSelectiveByIdRe(orderInfo);
		return ObjectRestResponse.succ();
	}

	/**
	 * 发货
	 *
	 * @return
	 */
	public ObjectRestResponse orderSend(ConfirmReceivedDto confirmReceivedDto) {
		if (confirmReceivedDto == null || confirmReceivedDto.getOrderId() == null || StringUtils.isBlank(confirmReceivedDto.getTrackingNumber())) {
			return ObjectRestResponse.paramIsEmpty();
		}
		OrderInfo orderInfo = selectById(confirmReceivedDto.getOrderId());
		if (orderInfo == null || orderInfo.getStatus().equals(OrderStatusEnum.ORDER_DELETE.getType())) {
			return ObjectRestResponse.createFailedResult(ResultCode.ORDER_NOT_EXIST, ResultCode.getMsg(ResultCode.ORDER_NOT_EXIST));
		}
		if (!orderInfo.getStatus().equals(OrderStatusEnum.ORDER_PAID.getType())) {
			return ObjectRestResponse.createFailedResult(ResultCode.FAILED_CODE, "订单状态错误");
		}
		orderInfo.setShippingStatus(OrderShippingStatusEnum.ORDER_SEND.getType());
		orderInfo.setStatus(OrderStatusEnum.ORDER_SEND.getType());
		orderInfo.setTrackingNumber(confirmReceivedDto.getTrackingNumber());
		orderInfo.setTrackingImg(confirmReceivedDto.getTrackingImg());
		updateSelectiveByIdRe(orderInfo);
		return ObjectRestResponse.succ();
	}

	/**
	 * 开发票
	 *
	 * @return
	 */
	public ObjectRestResponse setInvoice(OrderInvoiceDto orderInvoiceDto) {
		if (orderInvoiceDto == null || orderInvoiceDto.getOrderId() == null || StringUtils.isBlank(orderInvoiceDto.getInvoiceUrl())) {
			return ObjectRestResponse.paramIsEmpty();
		}
		OrderInfo orderInfo = selectById(orderInvoiceDto.getOrderId());
		if (orderInfo == null || orderInfo.getStatus().equals(OrderStatusEnum.ORDER_DELETE.getType())) {
			return ObjectRestResponse.createFailedResult(ResultCode.ORDER_NOT_EXIST, ResultCode.getMsg(ResultCode.ORDER_NOT_EXIST));
		}
		if (orderInfo.getInvoiceId() != null) {
			OrderEInvoice orderEInvoice = orderEInvoiceBiz.selectById(orderInfo.getInvoiceId());
			if (orderEInvoice == null) {
				return ObjectRestResponse.createFailedResult(ResultCode.FAILED_CODE, "订单发票不存在");
			}
			orderEInvoice.setStatus(OrderInvoiceStatusEnum.INVOICE_OPEN.getType());
			orderEInvoice.setInvoiceUrl(orderInvoiceDto.getInvoiceUrl());
			orderEInvoiceBiz.updateSelectiveByIdRe(orderEInvoice);

			orderInfo.setInvoiceStatus(OrderInvoiceStatusEnum.INVOICE_OPEN.getType());
			updateSelectiveByIdRe(orderInfo);
		} else {
			return ObjectRestResponse.createFailedResult(ResultCode.FAILED_CODE, "订单未开票");
		}
		return ObjectRestResponse.succ();
	}

	/**
	 * 发票发货
	 *
	 * @param confirmReceivedDto
	 * @return
	 */
	public ObjectRestResponse invoiceSend(ConfirmReceivedDto confirmReceivedDto) {
		if (confirmReceivedDto == null || confirmReceivedDto.getOrderId() == null || StringUtils.isBlank(confirmReceivedDto.getTrackingNumber())) {
			return ObjectRestResponse.paramIsEmpty();
		}
		OrderInfo orderInfo = selectById(confirmReceivedDto.getOrderId());
		if (orderInfo == null || orderInfo.getStatus().equals(OrderStatusEnum.ORDER_DELETE.getType())) {
			return ObjectRestResponse.createFailedResult(ResultCode.ORDER_NOT_EXIST, ResultCode.getMsg(ResultCode.ORDER_NOT_EXIST));
		}

		if (!orderInfo.getInvoiceStatus().equals(OrderInvoiceStatusEnum.INVOICE_OPEN.getType())) {
			return ObjectRestResponse.createFailedResult(ResultCode.FAILED_CODE, "订单状态错误");
		}

		OrderEInvoice orderEInvoice = orderEInvoiceBiz.selectById(orderInfo.getInvoiceId());
		if (orderEInvoice == null) {
			return ObjectRestResponse.createFailedResult(ResultCode.FAILED_CODE, "订单发票不存在");
		}
		orderEInvoice.setTrackingNumber(confirmReceivedDto.getTrackingNumber());
		orderEInvoice.setTrackingImg(confirmReceivedDto.getTrackingImg());
		orderEInvoiceBiz.updateSelectiveByIdRe(orderEInvoice);

		orderInfo.setInvoiceStatus(OrderInvoiceStatusEnum.INVOICE_SEND.getType());
		updateSelectiveByIdRe(orderInfo);

		return ObjectRestResponse.succ();
	}

	/**
	 * 订单发票确认收货
	 *
	 * @param orderId
	 * @return
	 */
	public ObjectRestResponse confirmOrderInvoice(Long orderId) {
		if (orderId == null) {
			return ObjectRestResponse.paramIsEmpty();
		}
		OrderInfo orderInfo = selectById(orderId);
		if (orderInfo == null || orderInfo.getStatus().equals(OrderStatusEnum.ORDER_DELETE.getType())) {
			return ObjectRestResponse.createFailedResult(ResultCode.ORDER_NOT_EXIST, ResultCode.getMsg(ResultCode.ORDER_NOT_EXIST));
		}

		if (!orderInfo.getInvoiceStatus().equals(OrderInvoiceStatusEnum.INVOICE_SEND.getType())) {
			return ObjectRestResponse.createFailedResult(ResultCode.FAILED_CODE, "订单状态错误");
		}
		orderInfo.setInvoiceStatus(OrderInvoiceStatusEnum.INVOICE_RECEIVED.getType());
		updateSelectiveByIdRe(orderInfo);
		return ObjectRestResponse.succ();
	}

	/**
	 * 取消订单
	 *
	 * @param param
	 * @return
	 */
	public ObjectRestResponse cancelOrder(OrderInfo param) {
		if (param == null || param.getOrderId() == null) {
			return ObjectRestResponse.paramIsEmpty();
		}
		OrderInfo orderInfo = selectById(param.getOrderId());
		if (orderInfo == null || orderInfo.getStatus().equals(OrderStatusEnum.ORDER_DELETE.getType())) {
			return ObjectRestResponse.createFailedResult(ResultCode.ORDER_NOT_EXIST, ResultCode.getMsg(ResultCode.ORDER_NOT_EXIST));
		}
		//订单状态为待支付前, 直接改变状态
		if (orderInfo.getStatus() < 3) {
			orderInfo.setStatus(OrderStatusEnum.ORDER_CANCEL.getType());
		} else if (orderInfo.getStatus() == 3) { //订单已支付
			orderInfo.setStatus(OrderStatusEnum.ORDER_CANCEL.getType());
			//todo 执行退款操作

		} else if (orderInfo.getStatus() > 3) { //已发货订单不可取消
			return ObjectRestResponse.createFailedResult(ResultCode.FAILED_CODE, "订单不可取消");
		}
		if (StringUtils.isNotBlank(param.getRemark())) {
			orderInfo.setCancelRemark(param.getRemark());
		}
		updateSelectiveByIdRe(orderInfo);
		return ObjectRestResponse.succ();
	}

	/**
	 * 删除订单
	 *
	 * @param orderId
	 * @return
	 */
	public ObjectRestResponse deleteOrder(Long orderId) {
		if (orderId == null) {
			return ObjectRestResponse.paramIsEmpty();
		}
		OrderInfo orderInfo = selectById(orderId);
		if (orderInfo == null || orderInfo.getStatus().equals(OrderStatusEnum.ORDER_DELETE.getType())) {
			return ObjectRestResponse.createFailedResult(ResultCode.ORDER_NOT_EXIST, ResultCode.getMsg(ResultCode.ORDER_NOT_EXIST));
		}
		orderInfo.setStatus(OrderStatusEnum.ORDER_DELETE.getType());
		updateSelectiveByIdRe(orderInfo);
		return ObjectRestResponse.succ();
	}


	/**
	 * 查询用户所有订单
	 *
	 * @param userOrderDto
	 * @return
	 */
	public ObjectRestResponse getOrderList(UserOrderDto userOrderDto) {
		Query query = new Query(userOrderDto);
		Example example = new Example(OrderInfo.class);
		Example orderItemExample = new Example(OrderItem.class);
		Example.Criteria orderInfoCriteria = example.createCriteria();
		Example.Criteria orderItemCriteria = orderItemExample.createCriteria();
		if (userOrderDto.getStatus() != null) {
			orderInfoCriteria.andEqualTo("status", userOrderDto.getStatus());
		}
		if (userOrderDto.getPayStatus() != null) {
			orderInfoCriteria.andEqualTo("payStatus", userOrderDto.getPayStatus());
		}
		if (userOrderDto.getUserId() != null) {
			orderInfoCriteria.andEqualTo("userId", userOrderDto.getUserId());
		}
		if (StringUtils.isNotBlank(userOrderDto.getOrderNo())) {
			orderInfoCriteria.andEqualTo("orderNo", userOrderDto.getOrderNo());
		}
		if (StringUtils.isNotBlank(userOrderDto.getNickname())) {
			orderInfoCriteria.andLike("nickname", "%" + userOrderDto.getNickname() + "%");
		}
		if (StringUtils.isNotBlank(userOrderDto.getPhone())) {
			orderInfoCriteria.andEqualTo("phone", userOrderDto.getPhone());
		}

		if (StringUtils.isNotBlank(userOrderDto.getName())) {
			orderItemCriteria.andLike("name", "%" + userOrderDto.getName() + "%");
		}
		orderInfoCriteria.andNotEqualTo("status", -1);
		example.orderBy("updTime").desc();
		PageDataVO<OrderInfo> pageDataVO = PageDataVO.pageInfo(query, () -> mapper.selectByExample(example));
		if (pageDataVO != null && pageDataVO.getData() != null) {
			fillOrderItem(pageDataVO.getData());
			fillOrderEInvoice(pageDataVO.getData());
			return ObjectRestResponse.succ(pageDataVO);
		}
		return ObjectRestResponse.succ(new PageDataVO<>());
	}


	/**
	 * 定制信息转订单
	 *
	 * @param customerId
	 * @return
	 */
	public ObjectRestResponse customerToOrderInfo(Integer customerId) {

		if (customerId == null) {
			return ObjectRestResponse.paramIsEmpty();
		}
		CustomForm customForm = customFormBiz.selectById(customerId);
		if (customForm == null) {
			return ObjectRestResponse.createFailedResult(ResultCode.NOTEXIST_CODE, ResultCode.getMsg(ResultCode.NOTEXIST_CODE));
		}
		AppUserDTO appUserDTO = userFeign.userDetailById(customForm.getUserId()).getData();
		if (appUserDTO == null) {
			return ObjectRestResponse.createFailedResult(ResultCode.USER_NOTEXIST_CODE, ResultCode.getMsg(ResultCode.USER_NOTEXIST_CODE));
		}
		OrderInfo orderInfo = new OrderInfo();
		orderInfo.setRealName(appUserDTO.getRealname());
		orderInfo.setUserId(appUserDTO.getUserid());
		orderInfo.setPhone(appUserDTO.getUsername());
		orderInfo.setNickname(appUserDTO.getNickname());
		orderInfo.setHeadImg(appUserDTO.getHeadimgurl());
		orderInfo.setStatus(OrderStatusEnum.ORDER_NO_PRICE.getType());
		orderInfo.setCustomerId(customerId);
		orderInfo.setCustomerJson(JSONObject.toJSONString(customForm));
		orderInfo.setNumber(1);
		orderInfo.setType(2);
		generateOrder(orderInfo);
		customForm.setOrderId(Integer.parseInt(String.valueOf(orderInfo.getOrderId())));
		//设置定制信息为已转成订单
		customForm.setStatus(2);
		customFormBiz.updateSelectiveByIdRe(customForm);
		return ObjectRestResponse.succ();
	}


	/**
	 * 设置定制订单价格
	 *
	 * @param orderInfo
	 * @return
	 */
	public ObjectRestResponse setOrderPrice(OrderInfo orderInfo) {
		if (orderInfo == null || orderInfo.getOrderId() == null || orderInfo.getAmount() == null) {
			return ObjectRestResponse.paramIsEmpty();
		}
		OrderInfo old = selectById(orderInfo.getOrderId());
		if (old == null) {
			return ObjectRestResponse.createFailedResult(ResultCode.ORDER_NOT_EXIST, ResultCode.getMsg(ResultCode.ORDER_NOT_EXIST));
		}
		old.setAmount(orderInfo.getAmount());
		old.setTotalAmount(orderInfo.getTotalAmount());
		updateSelectiveByIdRe(old);
		return ObjectRestResponse.succ();
	}

	/**
	 * 定制订单添加订单商品
	 *
	 * @param orderInfoDto
	 * @return
	 */
	public ObjectRestResponse setOrderItem(OrderInfoDto orderInfoDto) {
		if (orderInfoDto == null || orderInfoDto.getOrderId() == null) {
			return ObjectRestResponse.paramIsEmpty();
		}
		List<ItemInfoDto> itemInfoDtoList = orderInfoDto.getItemInfoDtoList();
		if (itemInfoDtoList == null || itemInfoDtoList.size() <= 0) {
			return ObjectRestResponse.paramIsEmpty();
		}
		OrderInfo orderInfo = selectById(orderInfoDto.getOrderId());
		if (orderInfo == null) {
			return ObjectRestResponse.createFailedResult(ResultCode.ORDER_NOT_EXIST, ResultCode.getMsg(ResultCode.ORDER_NOT_EXIST));
		}
		List<OrderItem> orderItemList = new ArrayList<>();
		itemInfoDtoList.parallelStream().forEach(itemInfoDto -> {
			switch (OrderTypeEnum.getByCode(itemInfoDto.getType())) {
				//标准数据
				case STANDARD_DATA:
					List<ImageDataVO> imageDataVOList = datacenterFeign.queryByIds(orderInfoDto.getDetailId() + "").getData();
					if (imageDataVOList != null && imageDataVOList.size() > 0) {
						ImageDataVO imageDataVO = imageDataVOList.get(0);
						OrderItem orderItem = new OrderItem();
						orderItem.setType(itemInfoDto.getType());
						orderItem.setTotalAmount(imageDataVO.getPrice().multiply(new BigDecimal(itemInfoDto.getNumber())));
						orderItem.setPrice(imageDataVO.getPrice());
						orderItem.setNumber(itemInfoDto.getNumber());
						orderItem.setDetailJson(JSONObject.toJSONString(imageDataVO));
						orderItem.setDetailId(Long.valueOf(imageDataVO.getId()));
						orderItemList.add(orderItem);
					}
					break;
				//行业应用
				case INDUSTRY_INFO:
					IndustryApplicationInfo industryApplicationInfo = industryApplicationInfoBiz.getOneById(itemInfoDto.getId().intValue()).getData();
					if (industryApplicationInfo != null) {
						OrderItem orderItem = new OrderItem();
						orderItem.setType(itemInfoDto.getType());
						orderItem.setTotalAmount(industryApplicationInfo.getPrice().multiply(new BigDecimal(itemInfoDto.getNumber())));
						orderItem.setPrice(industryApplicationInfo.getPrice());
						orderItem.setNumber(itemInfoDto.getNumber());
						orderItem.setName(industryApplicationInfo.getTitle());
						orderItem.setItemPic(industryApplicationInfo.getCoverImg());
						orderItem.setDetailJson(JSONObject.toJSONString(industryApplicationInfo));
						orderItem.setDetailId(Long.valueOf(industryApplicationInfo.getId()));
						orderItemList.add(orderItem);
					}
					break;
				//影像图库
				case IMAGE_STORAGE:
					ImageInfoRelation imageInfoRelation = imageInfoRelationBiz.selectById(itemInfoDto.getId());
					if (imageInfoRelation != null) {
						ImageImgStorage imageImgStorage = imageImgStorageBiz.getDetail(itemInfoDto.getId().intValue()).getData();
						if (imageImgStorage != null) {
							List<ImageInfoRelation> imageInfoRelationList = new ArrayList<>();
							imageInfoRelationList.add(imageInfoRelation);
							imageImgStorage.setImageInfoRelationList(imageInfoRelationList);
							OrderItem orderItem = new OrderItem();
							orderItem.setType(itemInfoDto.getType());
							orderItem.setTotalAmount(imageInfoRelation.getPrice().multiply(new BigDecimal(itemInfoDto.getNumber())));
							orderItem.setPrice(imageInfoRelation.getPrice());
							orderItem.setNumber(itemInfoDto.getNumber());
							orderItem.setName(imageImgStorage.getName());
							orderItem.setItemPic(imageImgStorage.getCoverImg());
							orderItem.setDetailJson(JSONObject.toJSONString(imageImgStorage));
							orderItem.setDetailId(Long.valueOf(imageImgStorage.getId()));
							orderItemList.add(orderItem);
						}
					}
					break;
			}
		});
		orderItemList.parallelStream().forEach(orderItem -> {
			if (orderItem != null) {
				orderItem.setOrderId(orderInfoDto.getOrderId());
				orderItemBiz.insertSelective(orderItem);
			}
		});
		return ObjectRestResponse.succ();
	}

	/**
	 * 定制订单 添加发票和地址信息
	 * @param orderInfoDto
	 * @return
	 */
	public ObjectRestResponse setAddressOrInvoice(OrderInfoDto orderInfoDto) {
		if (orderInfoDto == null || orderInfoDto.getOrderId() == null) {
			return ObjectRestResponse.paramIsEmpty();
		}
		OrderInfo orderInfo = selectById(orderInfoDto.getOrderId());
		if (orderInfo == null) {
			return ObjectRestResponse.createFailedResult(ResultCode.ORDER_NOT_EXIST, ResultCode.getMsg(ResultCode.ORDER_NOT_EXIST));
		}
		if (orderInfoDto.getUserAddressId() != null) {
			UserAddress userAddress = userAddressBiz.selectById(orderInfoDto.getUserAddressId());
			if (userAddress != null) {
				orderInfo.setReceiveName(userAddress.getConsigneeName());
				orderInfo.setReceivePhone(userAddress.getConsigneePhone());
				orderInfo.setReceiveAddress(userAddress.getProvince() + userAddress.getCity() + userAddress.getTown() + userAddress.getConsigneeAddress());
			}
		}
		//订单发票信息
		if (orderInfoDto.getUserInvoiceId() != null) {
			UserInvoice userInvoice = userInvoiceBiz.selectById(orderInfoDto.getUserInvoiceId());
			if (userInvoice != null) {
				OrderEInvoice orderEInvoice = new OrderEInvoice();
				orderEInvoice.setAmount(orderInfo.getAmount());
				orderEInvoice.setType(1);
				orderEInvoice.setOrderIds(orderInfo.getOrderId() + "");
				orderEInvoice.setTitleType(userInvoice.getTitleType());
				orderEInvoice.setTitleName(userInvoice.getTitleName());
				orderEInvoice.setTaxCode(userInvoice.getTaxCode());
				orderEInvoice.setContentRemark(orderInfoDto.getRemark());
				orderEInvoice.setContentAccount(userInvoice.getOpenBank() + "  " + userInvoice.getAccount());
				orderEInvoice.setContentContact(orderInfo.getReceiveAddress() + "  " + orderInfo.getReceiveName() + "  " + orderInfo.getReceivePhone());
				//发票内容
				if (StringUtils.isNotBlank(orderInfoDto.getInvoiceContent())) {
					orderEInvoice.setContentTitle(orderInfoDto.getInvoiceContent());
				}
				orderEInvoiceBiz.addUpdate(orderEInvoice);
				orderInfo.setInvoiceStatus(2);
				orderInfo.setInvoiceId(orderEInvoice.getId());
			}
		}
		orderInfo.setSendType(orderInfoDto.getSendType());
		updateSelectiveByIdRe(orderInfo);
		return ObjectRestResponse.succ();
	}

}